@@ -1,5 +1,5 @@
 
-/* (C)opyleft 2001-2002 Frank DENIS <j@pureftpd.org> */
+/* (C)opyleft 2001-2004 Frank DENIS <j@pureftpd.org> */
 
 #include <config.h>
 
@@ -117,7 +117,7 @@ int puredb_open(PureDB * const db, const char *dbfile)
     if ((char *) (db->map = 
                   (unsigned char *) mmap(NULL, db->size, PROT_READ,
                                          MAP_FILE | MAP_SHARED, db->fd, 
-                                         (off_t) 0)) == MAP_FAILED) {
+                                         (off_t) 0)) == (char *) MAP_FAILED) {
         db->map = NULL;
     }
 #elif defined(HAVE_MAPVIEWOFFILE)
@@ -134,7 +134,7 @@ int puredb_open(PureDB * const db, const char *dbfile)
                 CloseHandle(fmh);
             }
         }
-    }	
+    }    
 #endif    
     if (read_memcmp(db, (puredb_u32_t) 0U, 
                     (const unsigned char *) PUREDB_VERSION, 
@@ -155,7 +155,7 @@ int puredb_find(PureDB * const db, const char * const tofind,
     puredb_u32_t hash0;
     puredb_u32_t hash1;
     puredb_u32_t hash1e;
-    puredb_u32_t nbslots;
+    puredb_u32_t lastslot;
     puredb_u32_t slotlo;
     puredb_u32_t slothi;
     puredb_u32_t sno;
@@ -184,15 +184,16 @@ int puredb_find(PureDB * const db, const char * const tofind,
     if (hash1 > db->size) {
         return -2;                     /* corrupted table */
     }
-    nbslots = (hash1e - hash1) / (sizeof(puredb_u32_t) + sizeof(puredb_u32_t));
-    if (nbslots <= 0U) {
+    lastslot = (hash1e - hash1) / (sizeof(puredb_u32_t) + sizeof(puredb_u32_t));
+    if (lastslot <= 0U) {
         return -2;                     /* corrupted table */        
     }
+    lastslot--;
 #if !defined(MINIMAL) && !defined(NO_BINARY_LOOKUP)
     slotlo = 0U;
-    slothi = nbslots - 1U;
+    slothi = lastslot;
     sno = slothi / 2U;    
-    while (slotlo < slothi) {
+    while (slotlo <= slothi) {
         if (read_be_long(db, hash1 + sno * 
                          (sizeof(puredb_u32_t) + sizeof(puredb_u32_t)),
                          &scanned_hash) < 0) {
@@ -215,15 +216,21 @@ int puredb_find(PureDB * const db, const char * const tofind,
             goto shortcut;
         }
         if (scanned_hash > hash) {
-            slothi = sno - 1U;
+            if (sno <= 0U) {
+                break;
+            }
+            slothi = sno - 1;
         } else {
-            slotlo = sno + 1U;
+            if (sno >= lastslot) {
+                break;
+            }
+            slotlo = sno + 1;
         }
         sno = (slothi + slotlo) / 2U;        
     }
     hash1 += sno * (sizeof(puredb_u32_t) + sizeof(puredb_u32_t));
 #endif
-    do {
+    for(;;) {
         if (read_be_long(db, hash1, &scanned_hash) < 0) {
             return -3;
         }        
@@ -264,8 +271,11 @@ int puredb_find(PureDB * const db, const char * const tofind,
         }
         trynext:
         hash1 += sizeof(puredb_u32_t) + sizeof(puredb_u32_t);
-        nbslots--;
-    } while (nbslots > 0U);
+        if (lastslot == 0U) {
+            break;
+        }
+        lastslot--;
+    }
     
     return -1;                     /* not found (end of table) */
 }
