@@ -25,8 +25,8 @@ typedef struct FTPWhoEntry_ {
     volatile off_t restartat;
     volatile off_t download_total_size;
     volatile off_t download_current_size;    
-    char account[9];
-#if defined(__OpenBSD__) || defined(__MicroBSD__)
+    char account[MAX_USER_LENGTH + 1U];
+#if defined(__OpenBSD__) || defined(__MicroBSD__) || defined(__ekkoBSD__)
     char filename[1024];
 #else
 # ifdef PAGE_SIZE
