@@ -70,7 +70,7 @@ static char *pw_mysql_escape_string(MYSQL * const id_sql_server,
 #ifdef HAVE_MYSQL_REAL_ESCAPE_STRING
     tolen = mysql_real_escape_string(id_sql_server, to, from, from_len);
 #else
-/* MySQL 3.22.x and earlier are obsolete. Better use 3.23.x or 4.x */
+    /* MySQL 3.22.x and earlier are obsolete. Better use 3.23.x or 4.x */
     tolen = mysql_escape_string(to, from, from_len);    
 #endif
     if (tolen >= to_len || 
@@ -173,7 +173,7 @@ static char *sqlsubst(const char *orig_str, char * const query,
                 *query_pnt++ = '\\';
             }
         }
-        if (((unsigned char) *orig_str_scan) < 32U) {
+        if (ISCTRLCODE(*orig_str_scan)) {
             goto nextone;
         }
         if (--query_len <= (size_t) 0U) {
@@ -229,11 +229,12 @@ static char *pw_mysql_getquery(MYSQL * const id_sql_server,
                                const char * const peer_ip,
                                const char * const decimal_ip)
 {
+    char query[MYSQL_MAX_REQUEST_LENGTH];    
     MYSQL_RES *qresult = NULL;
     unsigned long *lengths;
+    char *answer = NULL;    
     MYSQL_ROW row;
-    char *answer = NULL;
-    char query[MYSQL_MAX_REQUEST_LENGTH];
+    size_t length;
     
     if (orig_query == NULL || *orig_query == 0) {
         goto bye;
@@ -259,14 +260,15 @@ static char *pw_mysql_getquery(MYSQL * const id_sql_server,
         goto bye;
     }
     lengths = mysql_fetch_lengths(qresult);
-    if (lengths == NULL || lengths[0] <= 0U) {
+    if (lengths == NULL || 
+        (length = (size_t) lengths[0] + (size_t) 1U) <= (size_t) 1U) {
         goto bye;
     }
-    if ((answer = malloc(lengths[0] + 1U)) == NULL) {
+    if ((answer = malloc(length)) == NULL) {
         goto bye;
     }
-    strncpy(answer, row[0], lengths[0]);
-    answer[lengths[0]] = 0;
+    strncpy(answer, row[0], length - (size_t) 1U);
+    answer[length - (size_t) 1U] = 0;
     
     bye:
     if (qresult != NULL) {
@@ -283,8 +285,8 @@ void pw_mysql_check(AuthResult * const result,
 {
     MYSQL *id_sql_server = NULL;
     const char *spwd = NULL;           /* stored password */
-    const char *uid = NULL;            /* stored system login/uid */
-    const char *gid = NULL;            /* stored system group/gid */
+    const char *uid = sql_default_uid; /* stored system login/uid */
+    const char *gid = sql_default_gid; /* stored system group/gid */
     const char *dir = NULL;            /* stored home directory */
 #ifdef QUOTAS
     const char *sqta_fs = NULL;        /* stored quota files */    
@@ -303,7 +305,6 @@ void pw_mysql_check(AuthResult * const result,
     char *escaped_port = NULL;
     char *escaped_peer_ip = NULL;
     char *escaped_decimal_ip = NULL;    
-    char *scrambled_password = NULL;
     int committed = 1;
     int crypto_crypt = 0, crypto_mysql = 0, crypto_md5 = 0, crypto_plain = 0;
     unsigned long decimal_ip_num = 0UL;
@@ -370,16 +371,22 @@ void pw_mysql_check(AuthResult * const result,
                                   escaped_decimal_ip)) == NULL) {
         goto bye;
     }
-    if ((uid = pw_mysql_getquery(id_sql_server, sqlreq_getuid,
-                                 escaped_account, escaped_ip, 
-                                 escaped_port, escaped_peer_ip,
-                                 escaped_decimal_ip)) == NULL) {
+    if (uid == NULL) {
+        uid = pw_mysql_getquery(id_sql_server, sqlreq_getuid,
+                                escaped_account, escaped_ip, 
+                                escaped_port, escaped_peer_ip,
+                                escaped_decimal_ip);
+    }
+    if (uid == NULL) {
         goto bye;
     }
-    if ((gid = pw_mysql_getquery(id_sql_server, sqlreq_getgid,
-                                 escaped_account, escaped_ip,
-                                 escaped_port, escaped_peer_ip,
-                                 escaped_decimal_ip)) == NULL) {
+    if (gid == NULL) {
+        gid = pw_mysql_getquery(id_sql_server, sqlreq_getgid,
+                                escaped_account, escaped_ip,
+                                escaped_port, escaped_peer_ip,
+                                escaped_decimal_ip);
+    }
+    if (gid == NULL) {
         goto bye;
     }
     if ((dir = pw_mysql_getquery(id_sql_server, sqlreq_getdir,
@@ -410,30 +417,19 @@ void pw_mysql_check(AuthResult * const result,
             goto auth_ok;
         }
     }
+#if MYSQL_VERSION_ID < 40100
     if (crypto_mysql != 0) {
-        unsigned int t;
-        unsigned char t1, t2;
+        unsigned long hash_res[2];
+        char scrambled_password[MYSQL_CRYPT_LEN];
         
-        if ((scrambled_password = 
-             malloc((size_t) (MYSQL_CRYPT_LEN + 2))) == NULL) {
-            goto bye;
-        }
-        t = zrand();
-        t1 = t & 0xff;
-        t2 = (t >> 8) & 0xff;
-        scrambled_password[MYSQL_CRYPT_LEN] = (char) t1;
-        scrambled_password[MYSQL_CRYPT_LEN + 1] = (char) t2;        
-        make_scrambled_password(scrambled_password, password);
-        if ((unsigned char) scrambled_password[MYSQL_CRYPT_LEN] != t1 ||
-            (unsigned char) scrambled_password[MYSQL_CRYPT_LEN + 1] != t2) {
-            for (;;) {
-                *scrambled_password++ = 0;
-            }
-        }
+        hash_password(hash_res, password);
+        snprintf(scrambled_password, sizeof scrambled_password, "%08lx%08lx", 
+                 hash_res[0], hash_res[1]);
         if (strcmp(scrambled_password, spwd) == 0) {
             goto auth_ok;
         }
     }
+#endif
     if (crypto_md5 != 0) {
         register const char *crypted;
 
@@ -546,8 +542,8 @@ void pw_mysql_check(AuthResult * const result,
             result->throttling_dl_changed = 1;
         }
     }
-#endif    
-    result->slow_tilde_expansion = 1;
+#endif
+    result->slow_tilde_expansion = !tildexp;
     result->auth_ok =- result->auth_ok;
     bye:
     if (committed == 0) {
@@ -556,66 +552,38 @@ void pw_mysql_check(AuthResult * const result,
     if (id_sql_server != NULL) {
         mysql_close(id_sql_server);
     }
-    if (spwd != NULL) {
-        free((void *) spwd);
-    }
-    if (uid != NULL) {
+    free((void *) spwd);
+    if (uid != sql_default_uid) {
         free((void *) uid);
     }
-    if (gid != NULL) {
-        free((void *) gid);
-    }
-    if (dir != NULL) {
-        free((void *) dir);
-    }
-    if (scrambled_password != NULL) {
-        free(scrambled_password);
+    if (gid != sql_default_gid) {
+        free((void *) gid); 
     }
+    free((void *) dir);
 #ifdef QUOTAS
-    if (sqta_fs != NULL) {
-        free((void *) sqta_fs);
-    }
-    if (sqta_sz != NULL) {
-        free((void *) sqta_sz);
-    }
+    free((void *) sqta_fs);
+    free((void *) sqta_sz);
 #endif    
 #ifdef RATIOS
-    if (ratio_ul != NULL) {
-        free((void *) ratio_ul);
-    }
-    if (ratio_dl != NULL) {
-        free((void *) ratio_dl);
-    }
+    free((void *) ratio_ul);
+    free((void *) ratio_dl);
 #endif    
 #ifdef THROTTLING
-    if (bandwidth_ul != NULL) {
-        free((void *) bandwidth_ul);
-    }
-    if (bandwidth_dl != NULL) {
-        free((void *) bandwidth_dl);
-    }
+    free((void *) bandwidth_ul);
+    free((void *) bandwidth_dl);
 #endif    
-    if (escaped_account != NULL) {
-        free((void *) escaped_account);
-    }
-    if (escaped_ip != NULL) {
-        free((void *) escaped_ip);
-    }
-    if (escaped_port != NULL) {
-        free((void *) escaped_port);
-    }
-    if (escaped_peer_ip != NULL) {
-        free((void *) escaped_peer_ip);
-    }
-    if (escaped_decimal_ip != NULL) {
-        free((void *) escaped_decimal_ip);
-    }    
+    free((void *) escaped_account);
+    free((void *) escaped_ip);
+    free((void *) escaped_port);
+    free((void *) escaped_peer_ip);
+    free((void *) escaped_decimal_ip);
 }
 
 void pw_mysql_parse(const char * const file)
 {
     if (generic_parser(file, mysql_config_keywords) != 0) {
-        die(421, LOG_ERR, MSG_CONF_ERR ": " MSG_ILLEGAL_CONFIG_FILE_SQL ": %s" , file);
+        die(421, LOG_ERR, MSG_CONF_ERR ": " MSG_ILLEGAL_CONFIG_FILE_SQL ": %s",
+        file);
     }    
     if (server == NULL && socket_path == NULL) {
         die(421, LOG_ERR, MSG_SQL_MISSING_SERVER);        
@@ -624,6 +592,13 @@ void pw_mysql_parse(const char * const file)
         free(socket_path);
         socket_path = NULL;
     }
+    if (tildexp_s != NULL) {
+    if ((tildexp = atoi(tildexp_s)) < 0) {
+        tildexp = 0;
+    }
+    free(tildexp_s);
+    tildexp_s = NULL;    
+    }        
     if (port_s != NULL) {
         port = atoi(port_s);
         if (port <= 0 || port > 65535) {
@@ -634,7 +609,7 @@ void pw_mysql_parse(const char * const file)
     }        
 }
 
-#define ZFREE(X) if ((X) != NULL) { free(X); (X) = NULL; }
+#define ZFREE(X) do { free(X); (X) = NULL; } while (0)
 
 void pw_mysql_exit(void)
 {
@@ -649,7 +624,9 @@ void pw_mysql_exit(void)
     ZFREE(transactions);
     ZFREE(sqlreq_getpw);
     ZFREE(sqlreq_getuid);
+    ZFREE(sql_default_uid);
     ZFREE(sqlreq_getgid);
+    ZFREE(sql_default_gid);
     ZFREE(sqlreq_getdir);
 #ifdef QUOTAS
     ZFREE(sqlreq_getqta_fs);
