@@ -173,18 +173,17 @@
 #define MSG_WELCOME_TO "Velkommen til"
 #define MSG_MAX_USERS "%lu brugere (det maksimale) er allerede logget p, desvrre"
 #define MSG_NB_USERS "Du er bruger nr %d af %d tilladte"
-#define MSG_WELCOME_TIME "Lokal tid er nu %02d:%02d og belastninen er %3.2f. Server port: %u."
+#define MSG_WELCOME_TIME "Lokal tid er nu %02d:%02d. Server port: %u."
 #define MSG_ANONYMOUS_FTP_ONLY "Kun anonym FTP er tilladt her"
 #define MSG_RATIOS_EVERYONE "FORHOLD ER SLET TIL FOR ALLE :"
 #define MSG_RATIOS_ANONYMOUS "ANONUME BRUGERE FR ET UL/DL FORHOLD :"
 #define MSG_RATIOS_RULE "for at downloade %u Mb, skal du uploade %u Mb godter."
 #define MSG_INFO_IDLE_M "Du bliver koblet af efter %lu minutters inaktivitet."
 #define MSG_INFO_IDLE_S "Du bliver koblet af efter %lu sekunders inaktivitet."
-#define MSG_INFO_CPU_TIME "Log af - CPU tid brugt: %ld.%03ld sekunder."
 #define MSG_CANT_READ_FILE "Kunne ikke lse [%s]"
 #define MSG_LS_TRUNCATED "Data afrundet til %u sammenfald"
 #define MSG_LS_SUCCESS "%u sammenfald totalt"
-#define MSG_LOGOUT "Logout"
+#define MSG_LOGOUT "Logout."
 #define MSG_AUTH_FAILED_LOG "Godkendelse mislykkedes for [%s]"
 #define MSG_ILLEGAL_UMASK "Forkert umask"
 #define MSG_STANDALONE_FAILED "Kunne ikke starte tandalone server"
@@ -194,11 +193,11 @@
 #define MSG_ACTIVE_DISABLED "Aktiv mode er slet fra"
 #define MSG_TRANSFER_SUCCESSFUL "Filoverfrsel lykkedes"
 #define MSG_NO_DISK_SPACE "Disken er fuld - upload venligst senere"
-#define MSG_OUT_OF_MEMORY "Er lbet tr for hukkommelse"
+#define MSG_OUT_OF_MEMORY "Er lbet tr for hukommelse"
 #define MSG_ILLEGAL_TRUSTED_IP "Ulovlig plidelig IP adresse"
 #define MSG_NO_ASCII_RESUME "ASCII genoptagning er usikker, slet venligst filen frst"
 #define MSG_UNKNOWN_ALTLOG "Ukendt log format"
-#define MSG_ACCOUNT_DISABLED "Du kan ikke logge p som [%s]: konto er slet fra"
+#define MSG_ACCOUNT_DISABLED "Du kan ikke logge p som [%s]: konto er deaktiveret"
 #define MSG_SQL_WRONG_PARMS "Forkerte SQL indstillinger"
 #define MSG_ILLEGAL_CONFIG_FILE_SQL "Forkert SQL konfigurations fil"
 #define MSG_SQL_MISSING_SERVER "Server i SQL konfigurationsfil mangler"
@@ -209,9 +208,13 @@
 #define MSG_QUOTA_EXCEEDED "Quota grnse overskredet : %s bliver ikke gemt"
 #define MSG_AUTH_UNKNOWN "Ukendt godkendelses metode"
 #define MSG_PDB_BROKEN "Kunne ikke lse den indexerede puredb fil (eller gammelt format fundet) - Prv pure-pw mkdb"
-#define MSG_ALIASES_ALIAS "%s is an alias for %s."
-#define MSG_ALIASES_UNKNOWN "Unknown alias %s."
-#define MSG_ALIASES_BROKEN_FILE "Damaged aliases file"
-#define MSG_ALIASES_LIST "The following aliases are available :"
-#define MSG_PERUSER_MAX "I can't accept more than %lu connections from the same user"
-#define MSG_IPV6_OK "IPv6 connections are also welcome on this server"
+#define MSG_ALIASES_ALIAS "%s er et alias for %s."
+#define MSG_ALIASES_UNKNOWN "Ukendt alias %s."
+#define MSG_ALIASES_BROKEN_FILE "delagt alias fil"
+#define MSG_ALIASES_LIST "Dess aliaser er tilgngelige :"
+#define MSG_PERUSER_MAX "Jeg kan ikke bne mere en %lu forbindelser for den samme bruger"
+#define MSG_IPV6_OK "IPv6 forbindelser er ogs velkomne p denne server"
+#define MSG_TLS_INFO "SSL/TLS: Enabled %s with %s, %d secret bits cipher"
+#define MSG_TLS_WEAK "SSL/TLS: Cipher too weak"
+#define MSG_TLS_NEEDED "Sorry, cleartext sessions are not accepted on this server.\n" \
+    "Please reconnect using SSL/TLS security mechanisms."
