@@ -29,9 +29,16 @@ static int upload_pipe_ropen(void)
 {
     struct stat st;
     int upload_pipe_fd;
-    
+    unsigned int tries = OPEN_TRIES;
+        
+    again:
     if ((upload_pipe_fd =
-         open(UPLOAD_PIPE_FILE, O_RDWR | O_NOFOLLOW)) == -1) {
+         open(UPLOAD_PIPE_FILE, O_RDONLY | O_NOFOLLOW)) == -1) {
+    if (tries > 0) {
+        tries--;
+        (void) sleep(OPEN_DELAY);
+        goto again;
+    }
         perror("Unable to open " UPLOAD_PIPE_FILE);
         return -1;
     }
@@ -66,8 +73,8 @@ static int readchar(const int upload_file_fd)
 static int readpipe(const int upload_file_fd,
                     char ** const r_who, char ** const r_file)
 {
-    static char who[WHOMAXLEN + 1U];
-    static char file[MAXPATHLEN + 1U];    
+    static char who[MAX_USER_LENGTH + 1U];
+    static char file[MAXPATHLEN + VHOST_PREFIX_MAX_LEN];
     const char * const whoend = &who[sizeof who];
     const char * const fileend = &file[sizeof file];
     char *whopnt = who;    
@@ -84,22 +91,22 @@ static int readpipe(const int upload_file_fd,
     } while (c != 2);
     while (whopnt != whoend) {
         c = readchar(upload_file_fd);
-        if (c == EOF || (c < 32 && c != 1)) {
+        if (c == EOF || (c != 1 && ISCTRLCODE(c))) {
             return -1;
         }
         if (c == 1) {
             *whopnt = 0;
             break;
         }
-        *whopnt = c;
+        *whopnt = (char) c;
         whopnt++;
     }    
     while (filepnt != fileend) {
         c = readchar(upload_file_fd);
-        if (c == EOF || (c < 32 && c != 0)) {
+        if (c == EOF || (c != 0 && ISCTRLCODE(c))) {
             return -1;
         }
-        *filepnt = c;
+        *filepnt = (char) c;
         if (c == 0) {
             break;
         }
@@ -118,7 +125,7 @@ static int readpipe(const int upload_file_fd,
 
 static char *checkvirtual(char *path)
 {
-    static char buf[MAXPATHLEN + 1U];
+    static char buf[MAXPATHLEN + 1];
     char *path_pnt;
     
     if (path == NULL || *path == '/' ||
@@ -156,8 +163,7 @@ static int closedesc_all(const int closestdin)
     (void) dup2(1, 2);
     if (fodder > 2) {
         (void) close(fodder);
-    }
-    
+    }    
     return 0;
 }
 
@@ -171,14 +177,15 @@ static void dodaemonize(void)
             return;
         } else if (child != (pid_t) 0) {
             _exit(EXIT_SUCCESS);
-        } else {
-            if (setsid() == (pid_t) -1) {
-               perror("Daemonization failed : setsid");
-            }
-            (void) chdir("/");
-            (void) closedesc_all(0);
+        } else if (setsid() == (pid_t) -1) {
+            perror("Daemonization failed : setsid");
         }
-    }    
+        (void) chdir("/");
+#ifdef HAVE_CLOSEFROM
+        (void) closefrom(3);
+#endif
+        (void) closedesc_all(0);
+    }
 }
 
 static int init(void)
@@ -228,7 +235,8 @@ static int parseoptions(int argc, char *argv[])
             break;
         }
         case 'g': {
-            char *nptr, *endptr;
+            const char *nptr;
+            char *endptr;
             
             nptr = optarg;
             endptr = NULL;
@@ -250,7 +258,8 @@ static int parseoptions(int argc, char *argv[])
             break;            
         }
         case 'u': {
-            char *nptr, *endptr;
+            const char *nptr;
+            char *endptr;
             
             nptr = optarg;
             endptr = NULL;
@@ -382,13 +391,18 @@ static int run(const char * const who, const char * const file,
             _exit(EXIT_FAILURE);
         }
         fillenv(who, &st);
-        execl(script, script, file, NULL);
+        execl(script, script, file, (char *) NULL);
         _exit(EXIT_FAILURE);
     } else if (pid != (pid_t) -1) {
 #ifdef HAVE_WAITPID
         (void) waitpid(pid, NULL, 0);
 #else
-        while (wait3(NULL, 0, NULL) != pid);
+        {
+            pid_t foundpid;
+            
+            while ((foundpid = wait3(NULL, 0, NULL)) != (pid_t) -1 &&
+                   foundpid != pid);
+        }
 #endif
     }
     
@@ -420,8 +434,8 @@ int safe_write(const int fd, const void *buf_, size_t count)
 
 static void updatepidfile(void)
 {
+    char buf[42];    
     int fd;
-    char buf[42];
     
     if (SNCHECK(snprintf(buf, sizeof buf, "%lu\n", 
                          (unsigned long) getpid()), sizeof buf)) {
@@ -477,7 +491,7 @@ int main(int argc, char *argv[])
     updatepidfile();
     if (changeuidgid() < 0) {
         perror("Identity change");
-        (void) unlink(PID_FILE);
+        (void) unlink(UPLOADSCRIPT_PID_FILE);
         return -1;
     }
 #ifdef SIGPIPE
@@ -488,6 +502,7 @@ int main(int argc, char *argv[])
 #endif
     for (;;) {
         if (readpipe(upload_pipe_fd, &who, &file) != 0) {
+            (void) sleep(1);
             continue;
         }        
         file = checkvirtual(file);
@@ -495,8 +510,11 @@ int main(int argc, char *argv[])
             run(who, file, upload_pipe_fd);
         }
     }
+    /* Unreachable */
+#if 0
     close(upload_pipe_fd);
-    (void) unlink(PID_FILE);
+    (void) unlink(UPLOADSCRIPT_PID_FILE);
+#endif
     
     return 0;
 }
